@@ -1,5 +1,5 @@
 #![allow(missing_docs)]
-use crate::style::{Color, OSControl, Style};
+use crate::style::{Color, Style};
 use crate::write::AnyWrite;
 use std::fmt;
 
@@ -13,59 +13,78 @@ impl Style {
             return Ok(());
         }
 
-        if self.has_sgr() {
-            // Prefix everything with reset characters if needed
-            if self.with_reset {
-                write!(f, "\x1B[0m")?
+        // Prefix everything with reset characters if needed
+        if self.prefix_with_reset {
+            write!(f, "\x1B[0m")?
+        }
+
+        // Write the codes’ prefix, then write numbers, separated by
+        // semicolons, for each text style we want to apply.
+        write!(f, "\x1B[")?;
+        let mut written_anything = false;
+
+        {
+            let mut write_char = |c| {
+                if written_anything {
+                    write!(f, ";")?;
+                }
+                written_anything = true;
+                #[cfg(feature = "gnu_legacy")]
+                write!(f, "0")?;
+                write!(f, "{}", c)?;
+                Ok(())
+            };
+
+            if self.is_bold {
+                write_char('1')?
+            }
+            if self.is_dimmed {
+                write_char('2')?
+            }
+            if self.is_italic {
+                write_char('3')?
+            }
+            if self.is_underline {
+                write_char('4')?
+            }
+            if self.is_blink {
+                write_char('5')?
+            }
+            if self.is_reverse {
+                write_char('7')?
+            }
+            if self.is_hidden {
+                write_char('8')?
+            }
+            if self.is_strikethrough {
+                write_char('9')?
+            }
+        }
+
+        // The foreground and background colors, if specified, need to be
+        // handled specially because the number codes are more complicated.
+        // (see `write_background_code` and `write_foreground_code`)
+
+        #[cfg(feature = "gnu_legacy")]
+        // with GNU, write foreground first, else background first.
+        {
+            if let Some(fg) = self.foreground {
+                if written_anything {
+                    write!(f, ";")?;
+                }
+                written_anything = true;
+                fg.write_foreground_code(f)?;
             }
 
-            // "Specified Graphical Rendition" prefixes
-            // Write the codes’ prefix, then write numbers, separated by
-            // semicolons, for each text style we want to apply.
-            write!(f, "\x1B[")?;
-            let mut written_anything = false;
-
-            {
-                let mut write_char = |c| {
-                    if written_anything {
-                        write!(f, ";")?;
-                    }
-                    written_anything = true;
-                    #[cfg(feature = "gnu_legacy")]
-                    write!(f, "0")?;
-                    write!(f, "{}", c)?;
-                    Ok(())
-                };
-
-                if self.is_bold {
-                    write_char('1')?
-                }
-                if self.is_dimmed {
-                    write_char('2')?
-                }
-                if self.is_italic {
-                    write_char('3')?
-                }
-                if self.is_underline {
-                    write_char('4')?
-                }
-                if self.is_blink {
-                    write_char('5')?
-                }
-                if self.is_reverse {
-                    write_char('7')?
-                }
-                if self.is_hidden {
-                    write_char('8')?
-                }
-                if self.is_strikethrough {
-                    write_char('9')?
+            if let Some(bg) = self.background {
+                if written_anything {
+                    write!(f, ";")?;
                 }
+                bg.write_background_code(f)?;
             }
-
-            // The foreground and background colors, if specified, need to be
-            // handled specially because the number codes are more complicated.
-            // (see `write_background_code` and `write_foreground_code`)
+        }
+        #[cfg(not(feature = "gnu_legacy"))]
+        {
             if let Some(bg) = self.background {
                 if written_anything {
                     write!(f, ";")?;
@@ -80,27 +99,9 @@ impl Style {
                 }
                 fg.write_foreground_code(f)?;
             }
-
-            // All the SGR codes end with an `m`, because reasons.
-            write!(f, "m")?;
-        }
-
-        // OS Control (OSC) prefixes
-        match self.oscontrol {
-            Some(OSControl::Hyperlink) => {
-                write!(f, "\x1B]8;;")?;
-            }
-            Some(OSControl::Title) => {
-                write!(f, "\x1B]2;")?;
-            }
-            Some(OSControl::Icon) => {
-                write!(f, "\x1B]I;")?;
-            }
-            Some(OSControl::Cwd) => {
-                write!(f, "\x1B]7;")?;
-            }
-            None => {}
         }
+        // All the codes end with an `m`, because reasons.
+        write!(f, "m")?;
 
         Ok(())
     }
@@ -110,17 +111,7 @@ impl Style {
         if self.is_plain() {
             Ok(())
         } else {
-            match self.oscontrol {
-                Some(OSControl::Hyperlink) => {
-                    write!(f, "{}{}", HYPERLINK_RESET, RESET)
-                }
-                Some(OSControl::Title) | Some(OSControl::Icon) | Some(OSControl::Cwd) => {
-                    write!(f, "{}", ST)
-                }
-                _ => {
-                    write!(f, "{}", RESET)
-                }
-            }
+            write!(f, "{}", RESET)
         }
     }
 }
@@ -128,10 +119,6 @@ impl Style {
 /// The code to send to reset all styles and return to `Style::default()`.
 pub static RESET: &str = "\x1B[0m";
 
-// The "String Termination" code. Used for OS Control (OSC) sequences.
-static ST: &str = "\x1B\\";
-pub(crate) static HYPERLINK_RESET: &str = "\x1B]8;;\x1B\\";
-
 impl Color {
     fn write_foreground_code<W: AnyWrite + ?Sized>(&self, f: &mut W) -> Result<(), W::Error> {
         match self {
@@ -396,33 +383,7 @@ impl fmt::Display for Infix {
             }
             Difference::Reset => {
                 let f: &mut dyn fmt::Write = f;
-
-                match (self.0, self.1) {
-                    (
-                        Style {
-                            oscontrol: Some(OSControl::Hyperlink),
-                            ..
-                        },
-                        Style {
-                            oscontrol: None, ..
-                        },
-                    ) => {
-                        write!(f, "{}{}", HYPERLINK_RESET, self.1.prefix())
-                    }
-                    (
-                        Style {
-                            oscontrol: Some(_), ..
-                        },
-                        Style {
-                            oscontrol: None, ..
-                        },
-                    ) => {
-                        write!(f, "{}{}", ST, self.1.prefix())
-                    }
-                    (_, _) => {
-                        write!(f, "{}{}", RESET, self.1.prefix())
-                    }
-                }
+                write!(f, "{}{}", RESET, self.1.prefix())
             }
             Difference::Empty => {
                 Ok(()) // nothing to write
@@ -457,6 +418,8 @@ macro_rules! test {
 mod test {
     use crate::style::Color::*;
     use crate::style::Style;
+    use crate::Color;
+    use std::default::Default;
 
     test!(plain:                 Style::default();                  "text/plain" => "text/plain");
     test!(red:                   Red;                               "hi" => "\x1B[31mhi\x1B[0m");
@@ -512,6 +475,19 @@ mod test {
         assert_eq!(White.normal().infix(Blue.normal()).to_string(), "\x1B[34m");
         assert_eq!(Blue.bold().infix(Blue.bold()).to_string(), "");
     }
+
+    #[test]
+    fn test_write_prefix_no_gnu_compat_order() {
+        let style = Style {
+            foreground: Some(Color::Red),
+            background: Some(Color::Blue),
+            ..Default::default()
+        };
+        assert_eq!(
+            style.paint("file").to_string(),
+            "\u{1b}[44;31mfile\u{1b}[0m".to_string()
+        );
+    }
 }
 
 #[cfg(test)]
@@ -519,6 +495,8 @@ mod test {
 mod gnu_legacy_test {
     use crate::style::Color::*;
     use crate::style::Style;
+    use crate::Color;
+    use std::default::Default;
 
     test!(plain:                 Style::default();                  "text/plain" => "text/plain");
     test!(red:                   Red;                               "hi" => "\x1B[31mhi\x1B[0m");
@@ -528,25 +506,24 @@ mod gnu_legacy_test {
     test!(blue_underline:        Blue.underline();                  "hi" => "\x1B[04;34mhi\x1B[0m");
     test!(green_bold_ul:         Green.bold().underline();          "hi" => "\x1B[01;04;32mhi\x1B[0m");
     test!(green_bold_ul_2:       Green.underline().bold();          "hi" => "\x1B[01;04;32mhi\x1B[0m");
-    test!(purple_on_white:       Purple.on(White);                  "hi" => "\x1B[47;35mhi\x1B[0m");
-    test!(purple_on_white_2:     Purple.normal().on(White);         "hi" => "\x1B[47;35mhi\x1B[0m");
-    test!(yellow_on_blue:        Style::new().on(Blue).fg(Yellow);  "hi" => "\x1B[44;33mhi\x1B[0m");
-    test!(yellow_on_blue_reset:  Cyan.on(Blue).reset_before_style().fg(Yellow); "hi" => "\x1B[0m\x1B[44;33mhi\x1B[0m");
-    test!(yellow_on_blue_reset_2: Cyan.on(Blue).fg(Yellow).reset_before_style(); "hi" => "\x1B[0m\x1B[44;33mhi\x1B[0m");
-    test!(magenta_on_white:      Magenta.on(White);                  "hi" => "\x1B[47;35mhi\x1B[0m");
-    test!(magenta_on_white_2:    Magenta.normal().on(White);         "hi" => "\x1B[47;35mhi\x1B[0m");
-    test!(yellow_on_blue_2:      Cyan.on(Blue).fg(Yellow);          "hi" => "\x1B[44;33mhi\x1B[0m");
-    test!(cyan_bold_on_white:    Cyan.bold().on(White);             "hi" => "\x1B[01;47;36mhi\x1B[0m");
-    test!(cyan_ul_on_white:      Cyan.underline().on(White);        "hi" => "\x1B[04;47;36mhi\x1B[0m");
-    test!(cyan_bold_ul_on_white: Cyan.bold().underline().on(White); "hi" => "\x1B[01;04;47;36mhi\x1B[0m");
-    test!(cyan_ul_bold_on_white: Cyan.underline().bold().on(White); "hi" => "\x1B[01;04;47;36mhi\x1B[0m");
+    test!(purple_on_white:       Purple.on(White);                  "hi" => "\x1B[35;47mhi\x1B[0m");
+    test!(purple_on_white_2:     Purple.normal().on(White);         "hi" => "\x1B[35;47mhi\x1B[0m");
+    test!(yellow_on_blue:        Style::new().on(Blue).fg(Yellow);  "hi" => "\x1B[33;44mhi\x1B[0m");
+    test!(yellow_on_blue_reset_2: Cyan.on(Blue).fg(Yellow).reset_before_style(); "hi" => "\x1B[0m\x1B[33;44mhi\x1B[0m");
+    test!(magenta_on_white:      Magenta.on(White);                  "hi" => "\x1B[35;47mhi\x1B[0m");
+    test!(magenta_on_white_2:    Magenta.normal().on(White);         "hi" => "\x1B[35;47mhi\x1B[0m");
+    test!(yellow_on_blue_2:      Cyan.on(Blue).fg(Yellow);          "hi" => "\x1B[33;44mhi\x1B[0m");
+    test!(cyan_bold_on_white:    Cyan.bold().on(White);             "hi" => "\x1B[01;36;47mhi\x1B[0m");
+    test!(cyan_ul_on_white:      Cyan.underline().on(White);        "hi" => "\x1B[04;36;47mhi\x1B[0m");
+    test!(cyan_bold_ul_on_white: Cyan.bold().underline().on(White); "hi" => "\x1B[01;04;36;47mhi\x1B[0m");
+    test!(cyan_ul_bold_on_white: Cyan.underline().bold().on(White); "hi" => "\x1B[01;04;36;47mhi\x1B[0m");
     test!(fixed:                 Fixed(100);                        "hi" => "\x1B[38;5;100mhi\x1B[0m");
-    test!(fixed_on_purple:       Fixed(100).on(Purple);             "hi" => "\x1B[45;38;5;100mhi\x1B[0m");
-    test!(fixed_on_fixed:        Fixed(100).on(Fixed(200));         "hi" => "\x1B[48;5;200;38;5;100mhi\x1B[0m");
+    test!(fixed_on_purple:       Fixed(100).on(Purple);             "hi" => "\x1B[38;5;100;45mhi\x1B[0m");
+    test!(fixed_on_fixed:        Fixed(100).on(Fixed(200));         "hi" => "\x1B[38;5;100;48;5;200mhi\x1B[0m");
     test!(rgb:                   Rgb(70,130,180);                   "hi" => "\x1B[38;2;70;130;180mhi\x1B[0m");
-    test!(rgb_on_blue:           Rgb(70,130,180).on(Blue);          "hi" => "\x1B[44;38;2;70;130;180mhi\x1B[0m");
-    test!(blue_on_rgb:           Blue.on(Rgb(70,130,180));          "hi" => "\x1B[48;2;70;130;180;34mhi\x1B[0m");
-    test!(rgb_on_rgb:            Rgb(70,130,180).on(Rgb(5,10,15));  "hi" => "\x1B[48;2;5;10;15;38;2;70;130;180mhi\x1B[0m");
+    test!(rgb_on_blue:           Rgb(70,130,180).on(Blue);          "hi" => "\x1B[38;2;70;130;180;44mhi\x1B[0m");
+    test!(blue_on_rgb:           Blue.on(Rgb(70,130,180));          "hi" => "\x1B[34;48;2;70;130;180mhi\x1B[0m");
+    test!(rgb_on_rgb:            Rgb(70,130,180).on(Rgb(5,10,15));  "hi" => "\x1B[38;2;70;130;180;48;2;5;10;15mhi\x1B[0m");
     test!(bold:                  Style::new().bold();               "hi" => "\x1B[01mhi\x1B[0m");
     test!(bold_with_reset:       Style::new().reset_before_style().bold(); "hi" => "\x1B[0m\x1B[01mhi\x1B[0m");
     test!(bold_with_reset_2:     Style::new().bold().reset_before_style(); "hi" => "\x1B[0m\x1B[01mhi\x1B[0m");
@@ -558,5 +535,18 @@ mod gnu_legacy_test {
     test!(reverse:               Style::new().reverse();            "hi" => "\x1B[07mhi\x1B[0m");
     test!(hidden:                Style::new().hidden();             "hi" => "\x1B[08mhi\x1B[0m");
     test!(stricken:              Style::new().strikethrough();      "hi" => "\x1B[09mhi\x1B[0m");
-    test!(lr_on_lr:              LightRed.on(LightRed);             "hi" => "\x1B[101;91mhi\x1B[0m");
+    test!(lr_on_lr:              LightRed.on(LightRed);             "hi" => "\x1B[91;101mhi\x1B[0m");
+
+    #[test]
+    fn test_write_prefix_gnu_compat_order() {
+        let style = Style {
+            foreground: Some(Color::Red),
+            background: Some(Color::Blue),
+            ..Default::default()
+        };
+        assert_eq!(
+            style.paint("file").to_string(),
+            "\u{1b}[31;44mfile\u{1b}[0m".to_string()
+        );
+    }
 }
